package jp.ac.osaka_u.ist.sel.clonedetector;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;

import jp.ac.osaka_u.ist.sel.clonedetector.data.ClonePair;
import jp.ac.osaka_u.ist.sel.clonedetector.data.CloneSet;
import jp.ac.osaka_u.ist.sel.clonedetector.data.Method;

public class LSHController {
	
	private final static String LSH_PATH="LSH\\LSHMain.exe";
	
	
	/**
	 * <p>p[^vZ</p>
	 */
	public static void computeParam(){
		String[] fromArray= { 
				LSH_PATH, 
				Integer.toString(CloneDetector.methodList.size()),
				Integer.toString(CloneDetector.methodList.size()),
				Integer.toString(CloneDetector.wordMap.size()),
				Double.toString(Params.LSH_PROB),Double.toString(Params.LSH_R),
				CloneDetector.DATASET_FILE,
				CloneDetector.DATASET_FILE,
				"3978000000",
				"-c"
			};
			ProcessBuilder pb = new ProcessBuilder(fromArray);
			try {			
				PrintWriter writer=new PrintWriter(new FileOutputStream(CloneDetector.PARAM_FILE));
				Process p = pb.start();
				InputStream errIn = p.getErrorStream();
				InputStream stdIn = p.getInputStream();
				int c;
				while ((c = stdIn.read()) != -1) 
				    writer.print((char)c);			
				stdIn.close();
				while ((c = errIn.read()) != -1)
				    ;//System.out.print((char)c);		        
			    errIn.close();
				p.waitFor();
				writer.close();
			} catch (InterruptedException | IOException e) {
				e.printStackTrace();
			}		
	}
	
	public static void computeParamFalse(){		
		try {
			PrintWriter writer=new PrintWriter(new FileOutputStream(CloneDetector.PARAM_FILE));
			writer.println("1");
			writer.println("R");
			writer.println(Double.toString(Params.LSH_R));
			writer.println("Success probability");
			writer.println(Double.toString(Params.LSH_PROB));
			writer.println("Dimension");
			writer.println(CloneDetector.wordMap.size());
			writer.println("R^2");
			writer.println(Double.toString(Params.LSH_R*Params.LSH_R));
			writer.println("Use <u> functions");
			writer.println("1");
			writer.println("k");
			writer.println("4");
			writer.println("m [# independent tuples of LSH functions]");
			writer.println("4");
			writer.println("L");
			writer.println("6");
			writer.println("W");
			writer.println("4.0000000000");
			writer.println("T");
			writer.println(CloneDetector.methodList.size());
			writer.println("typeHT");
			writer.println("3");
			writer.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}		
	}
	
	/**
	 * <p>LSH̎s</p>
	 * @throws InterruptedException
	 * @throws IOException
	 */
	public static void execute() {		
		String[] fromArray= { 
			LSH_PATH, 
			Integer.toString(CloneDetector.methodList.size()),
			Integer.toString(CloneDetector.methodList.size()),
			Integer.toString(CloneDetector.wordMap.size()),
			Double.toString(Params.LSH_PROB),Double.toString(Params.LSH_R),
			CloneDetector.DATASET_FILE,
			CloneDetector.DATASET_FILE,
			"3978000000",
			"-p",
			CloneDetector.PARAM_FILE,
		};
		

		ProcessBuilder pb = new ProcessBuilder(fromArray);
		try {			
			PrintWriter writer=new PrintWriter(new FileOutputStream(CloneDetector.LSH_FILE));
			Process p = pb.start();
			InputStream errIn = p.getErrorStream();
			InputStream stdIn = p.getInputStream();
			int c;
			while ((c = stdIn.read()) != -1) 
			    writer.print((char)c);			
			stdIn.close();
			while ((c = errIn.read()) != -1)
			    ;//System.out.print((char)c);		        
		    errIn.close();
			p.waitFor();
			writer.close();
		} catch (InterruptedException | IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * <p>N[yǍo</p>
	 * @return
	 * @throws IOException
	 */
	public static ArrayList<ClonePair> getClonePairList() throws IOException{
		ArrayList<ClonePair> clonePairList = new ArrayList<ClonePair>();
		BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(CloneDetector.LSH_FILE)));
		String line = null;
		int i=0;
		while((line=reader.readLine())!=null){
			if(line.contains("Query point")){			
				while(!(line=reader.readLine()).contains("Total time") && !line.contains("Mean query time")){					
					int methodId = Integer.valueOf(line.split("\t")[0]);
					//Double diff = Double.valueOf(line.split("\t")[1].replace("Distance:",""));
					if(i < methodId){						
						ClonePair pair = new ClonePair();
						pair.cloneA = CloneDetector.methodList.get(i);
						pair.cloneB = CloneDetector.methodList.get(methodId);		
						pair.sim = scalar(pair.cloneA.getVector(),pair.cloneB.getVector())/(pair.cloneA.getLen()*pair.cloneB.getLen());
						if(pair.sim>=Params.SIM_TH 
							&& Math.abs(pair.cloneA.getNodeNum()-pair.cloneB.getNodeNum()) < Params.DIFF_TH
							&& Math.abs(pair.cloneA.getLen() - pair.cloneB.getLen()) < Params.DIS_TH)
						clonePairList.add(pair);
					}					
				}
				i++;
			}			
		}				
		
	
		reader.close();		
		return clonePairList;
	}
	
	
	/**
	 * <p>N[Zbǧo</p>
	 * @return
	 * @throws IOException
	 */
	public static ArrayList<CloneSet> getCloneSetList(ArrayList<ClonePair> clonePairList) {
		
		for(int i=0; i<clonePairList.size(); i++){
	       	for(int j=i; j>0 && clonePairList.get(j).cloneB.getId() < clonePairList.get(j-1).cloneB.getId(); j--){
	       		ClonePair tmp = clonePairList.get(j);
	       		clonePairList.set(j,clonePairList.get(j-1));
	       		clonePairList.set(j-1,tmp);        		
	       	}        	        	
	       
		}
		
		for(int i=0; i<clonePairList.size(); i++){
	       	for(int j=i; j>0 && clonePairList.get(j).cloneA.getId() < clonePairList.get(j-1).cloneA.getId(); j--){
	       		ClonePair tmp = clonePairList.get(j);
	       		clonePairList.set(j,clonePairList.get(j-1));
	       		clonePairList.set(j-1,tmp);        		
	       	}        	        	
	       
		}
		
		ArrayList<CloneSet> cloneSetList = new ArrayList<CloneSet>();		
		for(ClonePair pair:clonePairList){			
			boolean addFlg = false;
			for(CloneSet set:cloneSetList){
				boolean contA = false;
				boolean contB = false;
				for(Method clone: set.cloneList){					
					if(clone.getClassName().equals(pair.cloneA.getClassName()) 
						&& clone.getStartLine()==pair.cloneA.getStartLine()){
						contA = true;
					}
					if(clone.getClassName().equals(pair.cloneB.getClassName()) 
						&& clone.getStartLine()==pair.cloneB.getStartLine()){
						contB = true;
					}					
				}				
				if(contA && contB){
					addFlg = true;
					break;
				}else if(contA && !contB){
					addFlg = true;
					set.cloneList.add(pair.cloneB);
					break;
				}else if(!contA && contB){
					addFlg = true;
					set.cloneList.add(pair.cloneA);
					break;
				}
			}			
			if(!addFlg){
				CloneSet set = new CloneSet();
				set.cloneList.add(pair.cloneA);
				set.cloneList.add(pair.cloneB);	
				cloneSetList.add(set);
			}			
		}
		
		for(CloneSet set: cloneSetList)
			set.calculateMetric();
		
		return cloneSetList;

	}	
	
	
	private static double scalar(double[] v1, double[] v2){
		double s = 0;
		for(int i=0; i<v1.length; i++){
			s += v1[i]*v2[i];
		}
		return s;		
	}


}
