package jp.ac.osaka_u.ist.sel.clonedetector;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

import jp.ac.osaka_u.ist.sel.clonedetector.data.ClonePair;
import jp.ac.osaka_u.ist.sel.clonedetector.data.CloneSet;
import jp.ac.osaka_u.ist.sel.clonedetector.data.Method;

public class Outputter {

	/**
	 * <p>]po<p>
	 * @param cloneSetList
	 * @throws FileNotFoundException
	 */
	public static void outputCSVforCPP(ArrayList<ClonePair> clonePairList) throws FileNotFoundException{
 		PrintWriter writer=new PrintWriter(new FileOutputStream(CloneDetector.RESULT_CSV));
 		for(ClonePair pair: clonePairList){
 			Method cloneA = pair.cloneA;
 			Method cloneB = pair.cloneB;
 			writer.printf("%s,%s,%d,%d,%f,%f,%f,\r\n",
 				cloneA.getClassName()+"."+cloneA.getName(),
 				cloneB.getClassName()+"."+cloneB.getName(),
 				cloneA.getNodeNum(),
 				cloneB.getNodeNum(),
 				cloneA.getLen(),
 				cloneB.getLen(),
 				pair.sim
 			);
  		} 		
 		writer.close();
	}
	
	/**
	 * <p>po<p>
	 * @param cloneSetList
	 * @throws IOException 
	 */
	public static void outputTXTforCPP(ArrayList<ClonePair> clonePairList) throws IOException{
		PrintWriter writer=new PrintWriter(new FileOutputStream(CloneDetector.RESULT_FILE));
		BufferedReader reader=null;
		int id=0;
		for(ClonePair pair: clonePairList){
			writer.printf("***********************\r\n");
			writer.printf("clonepair:%d == %f \r\n",id++,pair.sim);
			writer.printf("-----------\r\n");
			writer.printf("filename : %s (%d-%d) \r\n",pair.cloneA.getClassName(),pair.cloneA.getStartLine(),pair.cloneA.getEndLine());
			writer.printf("methodname : %s (%d) \r\n",pair.cloneA.getName(), pair.cloneA.getNodeNum());
			reader = new BufferedReader(new FileReader(pair.cloneA.getClassName()));
			for(int i=1; i<=pair.cloneA.getEndLine(); i++){
				String line = reader.readLine();
				if(i>=pair.cloneA.getStartLine() && i<=pair.cloneA.getEndLine())
					writer.printf("%4d: %s\r\n",i, line);
			}
			reader.close();			
			writer.printf("-----------\r\n");
			writer.printf("filename : %s (%d-%d) \r\n",pair.cloneB.getClassName(),pair.cloneB.getStartLine(),pair.cloneB.getEndLine());
			writer.printf("methodname : %s (%d) \r\n",pair.cloneB.getName(), pair.cloneB.getNodeNum());
			reader = new BufferedReader(new FileReader(pair.cloneB.getClassName()));
			for(int i=1; i<=pair.cloneB.getEndLine(); i++){
				String line = reader.readLine();
				if(i>=pair.cloneB.getStartLine() && i<=pair.cloneB.getEndLine())
					writer.printf("%4d: %s\r\n",i, line);
			}
			reader.close();		
		} 		
		writer.close();
	}
	
	/**
	 * <p>CSVo<p>
	 * @param cloneSetList
	 * @throws FileNotFoundException
	 */
	public static void outputCloneSetCSVforCPP(ArrayList<CloneSet> cloneSetList) throws FileNotFoundException{
 		PrintWriter writer=new PrintWriter(new FileOutputStream(CloneDetector.RESULT_CSV));
 		int i=0;
 		for(CloneSet set: cloneSetList){
 			writer.printf("%d,%d,%d,%f\r\n",i++,set.cloneList.size(),set.over50Token,set.tokenAverage); 				
  		} 		
 		writer.close();
	}
	
	/**
	 * <p>po<p>
	 * @param cloneSetList
	 * @throws IOException 
	 */
	public static void outputCloneSetTXTforCPP(ArrayList<CloneSet> cloneSetList) throws IOException{
		PrintWriter writer=new PrintWriter(new FileOutputStream(CloneDetector.RESULT_FILE));
		BufferedReader reader=null;
		int id=0;
		for(CloneSet set: cloneSetList){
			
			writer.printf("=================================================\r\n");			
			writer.printf("cloneset:%d\r\n",id++);
			writer.printf("=================================================\r\n");
			for(Method clone: set.cloneList){
				writer.printf("------------------------------------\r\n");
				writer.printf("filename : %s ( line:%d-%d ) \r\n",clone.getClassName(),clone.getStartLine(),clone.getEndLine());
				writer.printf("methodname : %s ( token:%d ) \r\n",clone.getName(), clone.getNodeNum());
				writer.printf("-------------------------------------\r\n");
				reader = new BufferedReader(new FileReader(clone.getClassName()));
				for(int i=1; i<=clone.getEndLine()+10; i++){
					String line = reader.readLine();
					if(line==null)
						break;
					if(i>=clone.getStartLine()-1 && i<=clone.getEndLine()+10)
						writer.printf("%s\r\n",line);
				}
				reader.close();			
			}
		} 		
		writer.close();
	}
	
	/**
	 * <p>]po<p>
	 * @param cloneSetList
	 * @throws FileNotFoundException
	 */
	public static void outputCSVforJava(ArrayList<ClonePair> clonePairList) throws FileNotFoundException{
 		PrintWriter writer=new PrintWriter(new FileOutputStream(CloneDetector.RESULT_CSV));
 		for(ClonePair pair: clonePairList){
 			Method cloneA = pair.cloneA;
 			Method cloneB = pair.cloneB;
 			writer.printf("%s,%s,%d,%d,%f,%f,%f,\r\n",
 				cloneA.getClassName()+"."+cloneA.getName(),
 				cloneB.getClassName()+"."+cloneB.getName(),
 				cloneA.getNodeNum(),
 				cloneB.getNodeNum(),
 				cloneA.getLen(),
 				cloneB.getLen(),
 				pair.sim
 			);
  		} 		
 		writer.close();
	}
	
	/**
	 * <p>po<p>
	 * @param cloneSetList
	 * @throws FileNotFoundException
	 */
	public static void outputTXTforJava(ArrayList<ClonePair> clonePairList) throws FileNotFoundException{
		PrintWriter writer=new PrintWriter(new FileOutputStream(CloneDetector.RESULT_FILE));
		int id=0;
		for(ClonePair pair: clonePairList){
			writer.printf("***********************\r\n");
			writer.printf("clonepair:%d=%f\r\n",id++,pair.sim);
			writer.printf("-----------\r\n");
			writer.printf("%s:%d,%f\r\n",pair.cloneA.getClassName(),pair.cloneA.getNodeNum(),pair.cloneA.getLen());
			writer.println(pair.cloneA.getCode());
			writer.printf("-----------\r\n");
			writer.printf("%s:%d,%f\r\n",pair.cloneB.getClassName(),pair.cloneB.getNodeNum(),pair.cloneB.getLen());
			writer.println(pair.cloneB.getCode());	
		} 		
		writer.close();
	}

}
