package jp.ac.osaka_u.ist.sel.clonedetector.analyze;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import jp.ac.osaka_u.ist.sel.clonedetector.CloneDetector;
import jp.ac.osaka_u.ist.sel.clonedetector.data.Method;
import jp.ac.osaka_u.ist.sel.clonedetector.data.Word;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.InfixExpression;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.PrefixExpression;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.StructuralPropertyDescriptor;
import org.eclipse.jdt.core.dom.TypeDeclaration;

public class JavaAnalyzer2 {

	private ASTParser parser = ASTParser.newParser(AST.JLS4);	
	private ArrayList<String> allWordList = new ArrayList<String>();
	
	/**
	 * <p>RXgN^</p>
	 */
	public JavaAnalyzer2(){
		parser.setBindingsRecovery(true);
		parser.setStatementsRecovery(true);
		parser.setResolveBindings(true);
	}
	
	/**
	 * <p>PꃊXg̎擾</p>
	 * @return
	 */
	public ArrayList<String> getWordList(){
		return allWordList;
	}
		
	/**
	 * <p>fBNgT</p>
	 * @param file
	 * @throws IOException
	 */
	public void searchFile(File file) throws IOException{		
		if(file.isFile() && file.getName().endsWith(".java")){	
			parser.setSource(getCode(file));
			final CompilationUnit unit = (CompilationUnit) parser.createAST(null);		
	 		ASTNode node = unit.getRoot();
	 		extractMethod(null, node, file.getParent().replace("\\","."));			
		}else if(file.isDirectory()){
			File[] fileList = file.listFiles();
			for(File f: fileList)
				searchFile(f);
		}	
	}
	
	
	/**
	 * <p>\[XR[heLXg擾</p>
	 * @param file
	 * @return
	 * @throws IOException
	 */
	private char[] getCode(File file) throws IOException{		
		BufferedReader reader = new BufferedReader(new FileReader(file));
		String code = "";
		String line;
		while((line=reader.readLine())!=null)
			code = code + "\n" +line;
		reader.close();
		return code.toCharArray();		
	}
	
	/**
	 * <p>ASTe\bhAST\z</p>
	 * @param method
	 * @param node
	 * @param parent
	 * @param className 
	 */
	@SuppressWarnings("unchecked")
	private void extractMethod(Method method,ASTNode node, String className) {
		
		if(node.getNodeType()== ASTNode.JAVADOC)
			return;
			
		boolean methodDecFlg = false;
		
		//\bh錾̏ꍇ
		if(node.getNodeType()==ASTNode.TYPE_DECLARATION){
			TypeDeclaration n= ((TypeDeclaration )node);
			className = className+"."+n.getName().getIdentifier();
		}			
		
		if(method==null && node.getNodeType()==ASTNode.METHOD_DECLARATION ){
			methodDecFlg = true;
			method = new Method();
			MethodDeclaration n = (MethodDeclaration) node;
			method.setName(n.getName().getIdentifier());	
			method.setClassName(className);
			method.setCode(node.toString());			
			CloneDetector.methodList.add(method);
		}		
		

		if(method!=null){	
			method.incNodeNum();
			switch(node.getNodeType()){		
				
				case ASTNode.SIMPLE_NAME:
				case ASTNode.QUALIFIED_NAME:
				case ASTNode.SIMPLE_TYPE:
					if(node.toString().length()>2) separateIdentifier(node.toString(),method.getWordList());
					else addWord(method.getWordList(),"word_2",Word.WORD);
					break;					
				case ASTNode.IF_STATEMENT:
				case ASTNode.SWITCH_STATEMENT:
				case ASTNode.WHILE_STATEMENT:
				case ASTNode.FOR_STATEMENT:
				case ASTNode.RETURN_STATEMENT: 
				case ASTNode.EXPRESSION_STATEMENT: 
				case ASTNode.VARIABLE_DECLARATION_STATEMENT:	
				case ASTNode.TRY_STATEMENT:
				case ASTNode.ASSERT_STATEMENT:
				case ASTNode.DO_STATEMENT:				
				case ASTNode.EMPTY_STATEMENT:
				case ASTNode.LABELED_STATEMENT:
				case ASTNode.SYNCHRONIZED_STATEMENT:
				case ASTNode.THROW_STATEMENT:
				case ASTNode.TYPE_DECLARATION_STATEMENT:
				case ASTNode.CONTINUE_STATEMENT:
				case ASTNode.BREAK_STATEMENT:
				case ASTNode.ENHANCED_FOR_STATEMENT:		
					addWord(method.getWordList(), "st_"+ Integer.toString(node.getNodeType()), Word.STATEMENT);		
					break;				
				case ASTNode.INFIX_EXPRESSION :
					addWord(method.getWordList(), "infixop_"+ ((InfixExpression)node).getOperator().toString(), Word.OPERATOR);		
					break;		
				case ASTNode.PREFIX_EXPRESSION:
					addWord(method.getWordList(), "prefixop_"+ ((PrefixExpression)node).getOperator().toString(), Word.OPERATOR);		
					break;	
				case ASTNode.SINGLE_VARIABLE_DECLARATION:
					addWord(method.getWordList(), "type_"+ ((SingleVariableDeclaration)node).getType().toString(), Word.WORD);		
			}		
		}
		
		//qm[h̎擾
		List<StructuralPropertyDescriptor> properties= node.structuralPropertiesForType();
		ArrayList<ASTNode> children = new ArrayList<ASTNode>();
		for (Iterator<?> i=properties.iterator(); i.hasNext();) {
			StructuralPropertyDescriptor descriptor=(StructuralPropertyDescriptor)i.next();
	        if (descriptor.isChildProperty()) {
	        	if((ASTNode)node.getStructuralProperty(descriptor)!=null)
	        		children.add((ASTNode)node.getStructuralProperty(descriptor));	        	
	        } else if (descriptor.isChildListProperty()) {
	        	List<?extends ASTNode> tmpChildren= (List<? extends ASTNode>) node.getStructuralProperty(descriptor);
	        	for (Iterator<? extends ASTNode> j= tmpChildren.iterator();j.hasNext();) {
	                children.add(j.next());                
                }
	        }
		}		
		//qm[h
		for(ASTNode n:children){		
			extractMethod(method,n,className);
		}
		
		if(methodDecFlg){			
			method=null;
		}
	}
	
	/**
	 * <p>PꃊXg</p>
	 * @param identifier
	 * @param wordList
	 */
	private void separateIdentifier(String identifier, ArrayList<Word> wordList){
		String word = new String("");
 		for(char c: identifier.toCharArray()){
 			if( 'a'<=c && c<='z'){
 				word = word+c; 					
 			}else if('A'<=c && c<='Z'){
 				if(!word.isEmpty()){
 					if('A'<=word.charAt(word.length()-1) && word.charAt(word.length()-1)<='Z'){
 						word = word+c; 		
 					}else{
 						addWord(wordList,word.toLowerCase(),Word.WORD);
 						word = ""+c; 		 					
 					}
 				}else{
 					word=word+c; 
 				}
 			}else{
 				addWord(wordList,word.toLowerCase(),Word.WORD);		
				word="";
			} 
 		}
 		addWord(wordList,word.toLowerCase(),Word.WORD);	
	}
	
	/**
	 * <p>[hXgւ̒ǉ</p>
	 * @param wordList
	 * @param word
	 */
	private void addWord(ArrayList<Word> wordList, String newWord, int type){
		if(!newWord.isEmpty()){
			boolean addFlg = true;
			for(Word word: wordList){
				if(word.getName().equals(newWord)){
					addFlg = false;
					word.inc();
					break;
				}					
			}			
			if(addFlg){
				wordList.add(new Word(newWord,type));
			}
		}
	}
}
