package jp.ac.osaka_u.ist.sel.m_sano.extractor;

import static org.junit.Assert.*;

import java.util.ArrayList;

import jp.ac.osaka_u.ist.sel.y_yuuki.cn.Logger;
import jp.ac.osaka_u.ist.sel.y_yuuki.cn.data.Method;

import org.junit.Test;

public class CMethodExtractorTest {

	/**
	 * AntClassLoader
	 * @throws Exception
	 */
	@Test
	public void testExtractMethod01() throws Exception {
		Logger.init();

		CMethodExtractor jme = new CMethodExtractor("testfile/Application/toppers/src/arch/logtrace/trace_dump.c");
		ArrayList<Method> mdList = jme.extractMethod();

		assertEquals(29, mdList.size());

		assertEquals("trace_proret_str", mdList.get(0).getName());
		assertEquals("trace_dump.c", mdList.get(0).getModifiedName());
		assertEquals("(ProtectionReturnType ret)", mdList.get(0).getParams());
		assertEquals("(ProtectionReturnType)", mdList.get(0).getParamTypeOnly(false));
		assertEquals(125, mdList.get(0).getStartLine());
		assertEquals(1, mdList.get(0).getStartColumn());
		assertEquals(142, mdList.get(0).getEndLine());
		assertEquals(1, mdList.get(0).getEndColumn());

		assertEquals("trace_schtblstatus_str", mdList.get(1).getName());
		assertEquals("trace_dump.c", mdList.get(1).getModifiedName());
		assertEquals("(ScheduleTableStatusType status)", mdList.get(1).getParams());
		assertEquals("(ScheduleTableStatusType)", mdList.get(1).getParamTypeOnly(false));
		assertEquals(147, mdList.get(1).getStartLine());
		assertEquals(1, mdList.get(1).getStartColumn());
		assertEquals(173, mdList.get(1).getEndLine());
		assertEquals(1, mdList.get(1).getEndColumn());

		assertEquals("atk2_task_state_str", mdList.get(2).getName());
		assertEquals("trace_dump.c", mdList.get(2).getModifiedName());
		assertEquals("(TaskStateType state)", mdList.get(2).getParams());
		assertEquals("(TaskStateType)", mdList.get(2).getParamTypeOnly(false));
		assertEquals(178, mdList.get(2).getStartLine());
		assertEquals(1, mdList.get(2).getStartColumn());
		assertEquals(201, mdList.get(2).getEndLine());
		assertEquals(1, mdList.get(2).getEndColumn());

		assertEquals("trace_get_tskid", mdList.get(3).getName());
		assertEquals("trace_dump.c", mdList.get(3).getModifiedName());
		assertEquals("(uintptr info)", mdList.get(3).getParams());
		assertEquals("(uintptr)", mdList.get(3).getParamTypeOnly(false));
		assertEquals(260, mdList.get(3).getStartLine());
		assertEquals(1, mdList.get(3).getStartColumn());
		assertEquals(274, mdList.get(3).getEndLine());
		assertEquals(1, mdList.get(3).getEndColumn());

		assertEquals("trace_print_tskenter", mdList.get(4).getName());
		assertEquals("trace_dump.c", mdList.get(4).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(4).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(4).getParamTypeOnly(false));
		assertEquals(280, mdList.get(4).getStartLine());
		assertEquals(1, mdList.get(4).getStartColumn());
		assertEquals(316, mdList.get(4).getEndLine());
		assertEquals(1, mdList.get(4).getEndColumn());

		assertEquals("trace_print_tskleave", mdList.get(5).getName());
		assertEquals("trace_dump.c", mdList.get(5).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(5).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(5).getParamTypeOnly(false));
		assertEquals(322, mdList.get(5).getStartLine());
		assertEquals(1, mdList.get(5).getStartColumn());
		assertEquals(358, mdList.get(5).getEndLine());
		assertEquals(1, mdList.get(5).getEndColumn());

		assertEquals("trace_print_evententer", mdList.get(6).getName());
		assertEquals("trace_dump.c", mdList.get(6).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(6).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(6).getParamTypeOnly(false));
		assertEquals(364, mdList.get(6).getStartLine());
		assertEquals(1, mdList.get(6).getStartColumn());
		assertEquals(405, mdList.get(6).getEndLine());
		assertEquals(1, mdList.get(6).getEndColumn());

		assertEquals("trace_print_eventleave", mdList.get(7).getName());
		assertEquals("trace_dump.c", mdList.get(7).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(7).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(7).getParamTypeOnly(false));
		assertEquals(411, mdList.get(7).getStartLine());
		assertEquals(1, mdList.get(7).getStartColumn());
		assertEquals(454, mdList.get(7).getEndLine());
		assertEquals(1, mdList.get(7).getEndColumn());

		assertEquals("trace_print_resenter", mdList.get(8).getName());
		assertEquals("trace_dump.c", mdList.get(8).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(8).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(8).getParamTypeOnly(false));
		assertEquals(460, mdList.get(8).getStartLine());
		assertEquals(1, mdList.get(8).getStartColumn());
		assertEquals(487, mdList.get(8).getEndLine());
		assertEquals(1, mdList.get(8).getEndColumn());

		assertEquals("trace_print_resleave", mdList.get(9).getName());
		assertEquals("trace_dump.c", mdList.get(9).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(9).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(9).getParamTypeOnly(false));
		assertEquals(493, mdList.get(9).getStartLine());
		assertEquals(1, mdList.get(9).getStartColumn());
		assertEquals(518, mdList.get(9).getEndLine());
		assertEquals(1, mdList.get(9).getEndColumn());

		assertEquals("trace_print_almenter", mdList.get(10).getName());
		assertEquals("trace_dump.c", mdList.get(10).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(10).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(10).getParamTypeOnly(false));
		assertEquals(523, mdList.get(10).getStartLine());
		assertEquals(1, mdList.get(10).getStartColumn());
		assertEquals(591, mdList.get(10).getEndLine());
		assertEquals(1, mdList.get(10).getEndColumn());

		assertEquals("trace_print_almleave", mdList.get(11).getName());
		assertEquals("trace_dump.c", mdList.get(11).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(11).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(11).getParamTypeOnly(false));
		assertEquals(597, mdList.get(11).getStartLine());
		assertEquals(1, mdList.get(11).getStartColumn());
		assertEquals(683, mdList.get(11).getEndLine());
		assertEquals(1, mdList.get(11).getEndColumn());

		assertEquals("trace_print_schtblenter", mdList.get(12).getName());
		assertEquals("trace_dump.c", mdList.get(12).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(12).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(12).getParamTypeOnly(false));
		assertEquals(688, mdList.get(12).getStartLine());
		assertEquals(1, mdList.get(12).getStartColumn());
		assertEquals(737, mdList.get(12).getEndLine());
		assertEquals(1, mdList.get(12).getEndColumn());

		assertEquals("trace_print_schtblleave", mdList.get(13).getName());
		assertEquals("trace_dump.c", mdList.get(13).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(13).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(13).getParamTypeOnly(false));
		assertEquals(742, mdList.get(13).getStartLine());
		assertEquals(1, mdList.get(13).getStartColumn());
		assertEquals(789, mdList.get(13).getEndLine());
		assertEquals(1, mdList.get(13).getEndColumn());

		assertEquals("trace_print_intenter", mdList.get(14).getName());
		assertEquals("trace_dump.c", mdList.get(14).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(14).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(14).getParamTypeOnly(false));
		assertEquals(794, mdList.get(14).getStartLine());
		assertEquals(1, mdList.get(14).getStartColumn());
		assertEquals(837, mdList.get(14).getEndLine());
		assertEquals(1, mdList.get(14).getEndColumn());

		assertEquals("trace_print_intleave", mdList.get(15).getName());
		assertEquals("trace_dump.c", mdList.get(15).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(15).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(15).getParamTypeOnly(false));
		assertEquals(843, mdList.get(15).getStartLine());
		assertEquals(1, mdList.get(15).getStartColumn());
		assertEquals(888, mdList.get(15).getEndLine());
		assertEquals(1, mdList.get(15).getEndColumn());

		assertEquals("trace_print_sysenter", mdList.get(16).getName());
		assertEquals("trace_dump.c", mdList.get(16).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(16).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(16).getParamTypeOnly(false));
		assertEquals(894, mdList.get(16).getStartLine());
		assertEquals(1, mdList.get(16).getStartColumn());
		assertEquals(923, mdList.get(16).getEndLine());
		assertEquals(1, mdList.get(16).getEndColumn());

		assertEquals("trace_print_sysleave", mdList.get(17).getName());
		assertEquals("trace_dump.c", mdList.get(17).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(17).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(17).getParamTypeOnly(false));
		assertEquals(929, mdList.get(17).getStartLine());
		assertEquals(1, mdList.get(17).getStartColumn());
		assertEquals(974, mdList.get(17).getEndLine());
		assertEquals(1, mdList.get(17).getEndColumn());

		assertEquals("trace_print_kerenter", mdList.get(18).getName());
		assertEquals("trace_dump.c", mdList.get(18).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[],void(*outputc)(char8 c))", mdList.get(18).getParams());
		assertEquals("(TRACE*,uintptr[],void(*)(char8))", mdList.get(18).getParamTypeOnly(false));
		assertEquals(980, mdList.get(18).getStartLine());
		assertEquals(1, mdList.get(18).getStartColumn());
		assertEquals(1007, mdList.get(18).getEndLine());
		assertEquals(1, mdList.get(18).getEndColumn());

		assertEquals("trace_print_svcenter", mdList.get(19).getName());
		assertEquals("trace_dump.c", mdList.get(19).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[],void(*outputc)(char8 c))", mdList.get(19).getParams());
		assertEquals("(TRACE*,uintptr[],void(*)(char8))", mdList.get(19).getParamTypeOnly(false));
		assertEquals(1013, mdList.get(19).getStartLine());
		assertEquals(1, mdList.get(19).getStartColumn());
		assertEquals(1094, mdList.get(19).getEndLine());
		assertEquals(1, mdList.get(19).getEndColumn());

		assertEquals("trace_print_svcleave", mdList.get(20).getName());
		assertEquals("trace_dump.c", mdList.get(20).getModifiedName());
		assertEquals("(const TRACE*trace,uintptr info[])", mdList.get(20).getParams());
		assertEquals("(TRACE*,uintptr[])", mdList.get(20).getParamTypeOnly(false));
		assertEquals(1100, mdList.get(20).getStartLine());
		assertEquals(1, mdList.get(20).getStartColumn());
		assertEquals(1176, mdList.get(20).getEndLine());
		assertEquals(1, mdList.get(20).getEndColumn());

		assertEquals("trace_set_context", mdList.get(21).getName());
		assertEquals("trace_dump.c", mdList.get(21).getModifiedName());
		assertEquals("(PR_UNIT_TYPE type,ObjectIDType id)", mdList.get(21).getParams());
		assertEquals("(PR_UNIT_TYPE,ObjectIDType)", mdList.get(21).getParamTypeOnly(false));
		assertEquals(1183, mdList.get(21).getStartLine());
		assertEquals(1, mdList.get(21).getStartColumn());
		assertEquals(1188, mdList.get(21).getEndLine());
		assertEquals(1, mdList.get(21).getEndColumn());

		assertEquals("trace_push_context", mdList.get(22).getName());
		assertEquals("trace_dump.c", mdList.get(22).getModifiedName());
		assertEquals("(PR_UNIT_TYPE type,ObjectIDType id)", mdList.get(22).getParams());
		assertEquals("(PR_UNIT_TYPE,ObjectIDType)", mdList.get(22).getParamTypeOnly(false));
		assertEquals(1193, mdList.get(22).getStartLine());
		assertEquals(1, mdList.get(22).getStartColumn());
		assertEquals(1198, mdList.get(22).getEndLine());
		assertEquals(1, mdList.get(22).getEndColumn());

		assertEquals("trace_get_context", mdList.get(23).getName());
		assertEquals("trace_dump.c", mdList.get(23).getModifiedName());
		assertEquals("(uintptr*p_obj_type_str,uintptr*p_id)", mdList.get(23).getParams());
		assertEquals("(uintptr*,uintptr*)", mdList.get(23).getParamTypeOnly(false));
		assertEquals(1208, mdList.get(23).getStartLine());
		assertEquals(1, mdList.get(23).getStartColumn());
		assertEquals(1236, mdList.get(23).getEndLine());
		assertEquals(1, mdList.get(23).getEndColumn());

		assertEquals("trace_pop_context", mdList.get(24).getName());
		assertEquals("trace_dump.c", mdList.get(24).getModifiedName());
		assertEquals("(uintptr*p_obj_type_str,uintptr*p_id)", mdList.get(24).getParams());
		assertEquals("(uintptr*,uintptr*)", mdList.get(24).getParamTypeOnly(false));
		assertEquals(1242, mdList.get(24).getStartLine());
		assertEquals(1, mdList.get(24).getStartColumn());
		assertEquals(1247, mdList.get(24).getEndLine());
		assertEquals(1, mdList.get(24).getEndColumn());

		assertEquals("trace_change_context", mdList.get(25).getName());
		assertEquals("trace_dump.c", mdList.get(25).getModifiedName());
		assertEquals("(PR_UNIT_TYPE type,ObjectIDType id)", mdList.get(25).getParams());
		assertEquals("(PR_UNIT_TYPE,ObjectIDType)", mdList.get(25).getParamTypeOnly(false));
		assertEquals(1253, mdList.get(25).getStartLine());
		assertEquals(1, mdList.get(25).getStartColumn());
		assertEquals(1276, mdList.get(25).getEndLine());
		assertEquals(1, mdList.get(25).getEndColumn());

		assertEquals("trace_resume_context", mdList.get(26).getName());
		assertEquals("trace_dump.c", mdList.get(26).getModifiedName());
		assertEquals("(void)", mdList.get(26).getParams());
		assertEquals("(void)", mdList.get(26).getParamTypeOnly(false));
		assertEquals(1282, mdList.get(26).getStartLine());
		assertEquals(1, mdList.get(26).getStartColumn());
		assertEquals(1298, mdList.get(26).getEndLine());
		assertEquals(1, mdList.get(26).getEndColumn());

		assertEquals("trace_print", mdList.get(27).getName());
		assertEquals("trace_dump.c", mdList.get(27).getModifiedName());
		assertEquals("(const TRACE*p_trace,void(*outputc)(char8 c))", mdList.get(27).getParams());
		assertEquals("(TRACE*,void(*)(char8))", mdList.get(27).getParamTypeOnly(false));
		assertEquals(1303, mdList.get(27).getStartLine());
		assertEquals(1, mdList.get(27).getStartColumn());
		assertEquals(1493, mdList.get(27).getEndLine());
		assertEquals(1, mdList.get(27).getEndColumn());

		assertEquals("trace_dump", mdList.get(28).getName());
		assertEquals("trace_dump.c", mdList.get(28).getModifiedName());
		assertEquals("(void(*exinf)(char8 c))", mdList.get(28).getParams());
		assertEquals("(void(*)(char8))", mdList.get(28).getParamTypeOnly(false));
		assertEquals(1498, mdList.get(28).getStartLine());
		assertEquals(1, mdList.get(28).getStartColumn());
		assertEquals(1518, mdList.get(28).getEndLine());
		assertEquals(1, mdList.get(28).getEndColumn());

	}

	/**
	 * heaptuple
	 * @throws Exception
	 */
	@Test
	public void testExtractMethod02() throws Exception {
		Logger.init();

		CMethodExtractor jme = new CMethodExtractor("testfile/PostgreSQL/heaptuple.c");
		ArrayList<Method> mdList = jme.extractMethod();

		assertEquals(25, mdList.size());

		assertEquals("heap_compute_data_size", mdList.get(0).getName());
		assertEquals("heaptuple.c", mdList.get(0).getModifiedName());
		assertEquals("(TupleDesc tupleDesc,Datum*values,bool*isnull)", mdList.get(0).getParams());
		assertEquals("(TupleDesc,Datum*,bool*)", mdList.get(0).getParamTypeOnly(false));
		assertEquals(83, mdList.get(0).getStartLine());
		assertEquals(1, mdList.get(0).getStartColumn());
		assertEquals(121, mdList.get(0).getEndLine());
		assertEquals(1, mdList.get(0).getEndColumn());

		assertEquals("heap_fill_tuple", mdList.get(1).getName());
		assertEquals("heaptuple.c", mdList.get(1).getModifiedName());
		assertEquals("(TupleDesc tupleDesc,Datum*values,bool*isnull,char*data,Size data_size,uint16*infomask,bits8*bit)", mdList.get(1).getParams());
		assertEquals("(TupleDesc,Datum*,bool*,char*,Size,uint16*,bits8*)", mdList.get(1).getParamTypeOnly(false));
		assertEquals(132, mdList.get(1).getStartLine());
		assertEquals(1, mdList.get(1).getStartColumn());
		assertEquals(255, mdList.get(1).getEndLine());
		assertEquals(1, mdList.get(1).getEndColumn());

		assertEquals("heap_attisnull", mdList.get(2).getName());
		assertEquals("heaptuple.c", mdList.get(2).getModifiedName());
		assertEquals("(HeapTuple tup,int attnum)", mdList.get(2).getParams());
		assertEquals("(HeapTuple,int)", mdList.get(2).getParamTypeOnly(false));
		assertEquals(267, mdList.get(2).getStartLine());
		assertEquals(1, mdList.get(2).getStartColumn());
		assertEquals(297, mdList.get(2).getEndLine());
		assertEquals(1, mdList.get(2).getEndColumn());

		assertEquals("nocachegetattr", mdList.get(3).getName());
		assertEquals("heaptuple.c", mdList.get(3).getModifiedName());
		assertEquals("(HeapTuple tuple,int attnum,TupleDesc tupleDesc)", mdList.get(3).getParams());
		assertEquals("(HeapTuple,int,TupleDesc)", mdList.get(3).getParamTypeOnly(false));
		assertEquals(322, mdList.get(3).getStartLine());
		assertEquals(1, mdList.get(3).getStartColumn());
		assertEquals(511, mdList.get(3).getEndLine());
		assertEquals(1, mdList.get(3).getEndColumn());

		assertEquals("heap_getsysattr", mdList.get(4).getName());
		assertEquals("heaptuple.c", mdList.get(4).getModifiedName());
		assertEquals("(HeapTuple tup,int attnum,TupleDesc tupleDesc,bool*isnull)", mdList.get(4).getParams());
		assertEquals("(HeapTuple,int,TupleDesc,bool*)", mdList.get(4).getParamTypeOnly(false));
		assertEquals(522, mdList.get(4).getStartLine());
		assertEquals(1, mdList.get(4).getStartColumn());
		assertEquals(567, mdList.get(4).getEndLine());
		assertEquals(1, mdList.get(4).getEndColumn());

		assertEquals("heap_copytuple", mdList.get(5).getName());
		assertEquals("heaptuple.c", mdList.get(5).getModifiedName());
		assertEquals("(HeapTuple tuple)", mdList.get(5).getParams());
		assertEquals("(HeapTuple)", mdList.get(5).getParamTypeOnly(false));
		assertEquals(578, mdList.get(5).getStartLine());
		assertEquals(1, mdList.get(5).getStartColumn());
		assertEquals(593, mdList.get(5).getEndLine());
		assertEquals(1, mdList.get(5).getEndColumn());

		assertEquals("heap_copytuple_with_tuple", mdList.get(6).getName());
		assertEquals("heaptuple.c", mdList.get(6).getModifiedName());
		assertEquals("(HeapTuple src,HeapTuple dest)", mdList.get(6).getParams());
		assertEquals("(HeapTuple,HeapTuple)", mdList.get(6).getParamTypeOnly(false));
		assertEquals(604, mdList.get(6).getStartLine());
		assertEquals(1, mdList.get(6).getStartColumn());
		assertEquals(618, mdList.get(6).getEndLine());
		assertEquals(1, mdList.get(6).getEndColumn());

		assertEquals("heap_copy_tuple_as_datum", mdList.get(7).getName());
		assertEquals("heaptuple.c", mdList.get(7).getModifiedName());
		assertEquals("(HeapTuple tuple,TupleDesc tupleDesc)", mdList.get(7).getParams());
		assertEquals("(HeapTuple,TupleDesc)", mdList.get(7).getParamTypeOnly(false));
		assertEquals(626, mdList.get(7).getStartLine());
		assertEquals(1, mdList.get(7).getStartColumn());
		assertEquals(653, mdList.get(7).getEndLine());
		assertEquals(1, mdList.get(7).getEndColumn());

		assertEquals("heap_form_tuple", mdList.get(8).getName());
		assertEquals("heaptuple.c", mdList.get(8).getModifiedName());
		assertEquals("(TupleDesc tupleDescriptor,Datum*values,bool*isnull)", mdList.get(8).getParams());
		assertEquals("(TupleDesc,Datum*,bool*)", mdList.get(8).getParamTypeOnly(false));
		assertEquals(662, mdList.get(8).getStartLine());
		assertEquals(1, mdList.get(8).getStartColumn());
		assertEquals(746, mdList.get(8).getEndLine());
		assertEquals(1, mdList.get(8).getEndColumn());

		assertEquals("heap_formtuple", mdList.get(9).getName());
		assertEquals("heaptuple.c", mdList.get(9).getModifiedName());
		assertEquals("(TupleDesc tupleDescriptor,Datum*values,char*nulls)", mdList.get(9).getParams());
		assertEquals("(TupleDesc,Datum*,char*)", mdList.get(9).getParamTypeOnly(false));
		assertEquals(760, mdList.get(9).getStartLine());
		assertEquals(1, mdList.get(9).getStartColumn());
		assertEquals(778, mdList.get(9).getEndLine());
		assertEquals(1, mdList.get(9).getEndColumn());

		assertEquals("heap_modify_tuple", mdList.get(10).getName());
		assertEquals("heaptuple.c", mdList.get(10).getModifiedName());
		assertEquals("(HeapTuple tuple,TupleDesc tupleDesc,Datum*replValues,bool*replIsnull,bool*doReplace)", mdList.get(10).getParams());
		assertEquals("(HeapTuple,TupleDesc,Datum*,bool*,bool*)", mdList.get(10).getParamTypeOnly(false));
		assertEquals(792, mdList.get(10).getStartLine());
		assertEquals(1, mdList.get(10).getStartColumn());
		assertEquals(849, mdList.get(10).getEndLine());
		assertEquals(1, mdList.get(10).getEndColumn());

		assertEquals("heap_modifytuple", mdList.get(11).getName());
		assertEquals("heaptuple.c", mdList.get(11).getModifiedName());
		assertEquals("(HeapTuple tuple,TupleDesc tupleDesc,Datum*replValues,char*replNulls,char*replActions)", mdList.get(11).getParams());
		assertEquals("(HeapTuple,TupleDesc,Datum*,char*,char*)", mdList.get(11).getParamTypeOnly(false));
		assertEquals(862, mdList.get(11).getStartLine());
		assertEquals(1, mdList.get(11).getStartColumn());
		assertEquals(887, mdList.get(11).getEndLine());
		assertEquals(1, mdList.get(11).getEndColumn());

		assertEquals("heap_deform_tuple", mdList.get(12).getName());
		assertEquals("heaptuple.c", mdList.get(12).getModifiedName());
		assertEquals("(HeapTuple tuple,TupleDesc tupleDesc,Datum*values,bool*isnull)", mdList.get(12).getParams());
		assertEquals("(HeapTuple,TupleDesc,Datum*,bool*)", mdList.get(12).getParamTypeOnly(false));
		assertEquals(906, mdList.get(12).getStartLine());
		assertEquals(1, mdList.get(12).getStartColumn());
		assertEquals(994, mdList.get(12).getEndLine());
		assertEquals(1, mdList.get(12).getEndColumn());

		assertEquals("heap_deformtuple", mdList.get(13).getName());
		assertEquals("heaptuple.c", mdList.get(13).getModifiedName());
		assertEquals("(HeapTuple tuple,TupleDesc tupleDesc,Datum*values,char*nulls)", mdList.get(13).getParams());
		assertEquals("(HeapTuple,TupleDesc,Datum*,char*)", mdList.get(13).getParamTypeOnly(false));
		assertEquals(1018, mdList.get(13).getStartLine());
		assertEquals(1, mdList.get(13).getStartColumn());
		assertEquals(1034, mdList.get(13).getEndLine());
		assertEquals(1, mdList.get(13).getEndColumn());

		assertEquals("slot_deform_tuple", mdList.get(14).getName());
		assertEquals("heaptuple.c", mdList.get(14).getModifiedName());
		assertEquals("(TupleTableSlot*slot,int natts)", mdList.get(14).getParams());
		assertEquals("(TupleTableSlot*,int)", mdList.get(14).getParamTypeOnly(false));
		assertEquals(1047, mdList.get(14).getStartLine());
		assertEquals(1, mdList.get(14).getStartColumn());
		assertEquals(1140, mdList.get(14).getEndLine());
		assertEquals(1, mdList.get(14).getEndColumn());

		assertEquals("slot_getattr", mdList.get(15).getName());
		assertEquals("heaptuple.c", mdList.get(15).getModifiedName());
		assertEquals("(TupleTableSlot*slot,int attnum,bool*isnull)", mdList.get(15).getParams());
		assertEquals("(TupleTableSlot*,int,bool*)", mdList.get(15).getParamTypeOnly(false));
		assertEquals(1154, mdList.get(15).getStartLine());
		assertEquals(1, mdList.get(15).getStartColumn());
		assertEquals(1242, mdList.get(15).getEndLine());
		assertEquals(1, mdList.get(15).getEndColumn());

		assertEquals("slot_getallattrs", mdList.get(16).getName());
		assertEquals("heaptuple.c", mdList.get(16).getModifiedName());
		assertEquals("(TupleTableSlot*slot)", mdList.get(16).getParams());
		assertEquals("(TupleTableSlot*)", mdList.get(16).getParamTypeOnly(false));
		assertEquals(1250, mdList.get(16).getStartLine());
		assertEquals(1, mdList.get(16).getStartColumn());
		assertEquals(1287, mdList.get(16).getEndLine());
		assertEquals(1, mdList.get(16).getEndColumn());

		assertEquals("slot_getsomeattrs", mdList.get(17).getName());
		assertEquals("heaptuple.c", mdList.get(17).getModifiedName());
		assertEquals("(TupleTableSlot*slot,int attnum)", mdList.get(17).getParams());
		assertEquals("(TupleTableSlot*,int)", mdList.get(17).getParamTypeOnly(false));
		assertEquals(1294, mdList.get(17).getStartLine());
		assertEquals(1, mdList.get(17).getStartColumn());
		assertEquals(1334, mdList.get(17).getEndLine());
		assertEquals(1, mdList.get(17).getEndColumn());

		assertEquals("slot_attisnull", mdList.get(18).getName());
		assertEquals("heaptuple.c", mdList.get(18).getModifiedName());
		assertEquals("(TupleTableSlot*slot,int attnum)", mdList.get(18).getParams());
		assertEquals("(TupleTableSlot*,int)", mdList.get(18).getParamTypeOnly(false));
		assertEquals(1341, mdList.get(18).getStartLine());
		assertEquals(1, mdList.get(18).getStartColumn());
		assertEquals(1380, mdList.get(18).getEndLine());
		assertEquals(1, mdList.get(18).getEndColumn());

		assertEquals("heap_freetuple", mdList.get(19).getName());
		assertEquals("heaptuple.c", mdList.get(19).getModifiedName());
		assertEquals("(HeapTuple htup)", mdList.get(19).getParams());
		assertEquals("(HeapTuple)", mdList.get(19).getParamTypeOnly(false));
		assertEquals(1385, mdList.get(19).getStartLine());
		assertEquals(1, mdList.get(19).getStartColumn());
		assertEquals(1389, mdList.get(19).getEndLine());
		assertEquals(1, mdList.get(19).getEndColumn());

		assertEquals("heap_form_minimal_tuple", mdList.get(20).getName());
		assertEquals("heaptuple.c", mdList.get(20).getModifiedName());
		assertEquals("(TupleDesc tupleDescriptor,Datum*values,bool*isnull)", mdList.get(20).getParams());
		assertEquals("(TupleDesc,Datum*,bool*)", mdList.get(20).getParamTypeOnly(false));
		assertEquals(1403, mdList.get(20).getStartLine());
		assertEquals(1, mdList.get(20).getStartColumn());
		assertEquals(1475, mdList.get(20).getEndLine());
		assertEquals(1, mdList.get(20).getEndColumn());

		assertEquals("heap_free_minimal_tuple", mdList.get(21).getName());
		assertEquals("heaptuple.c", mdList.get(21).getModifiedName());
		assertEquals("(MinimalTuple mtup)", mdList.get(21).getParams());
		assertEquals("(MinimalTuple)", mdList.get(21).getParamTypeOnly(false));
		assertEquals(1480, mdList.get(21).getStartLine());
		assertEquals(1, mdList.get(21).getStartColumn());
		assertEquals(1484, mdList.get(21).getEndLine());
		assertEquals(1, mdList.get(21).getEndColumn());

		assertEquals("heap_copy_minimal_tuple", mdList.get(22).getName());
		assertEquals("heaptuple.c", mdList.get(22).getModifiedName());
		assertEquals("(MinimalTuple mtup)", mdList.get(22).getParams());
		assertEquals("(MinimalTuple)", mdList.get(22).getParamTypeOnly(false));
		assertEquals(1492, mdList.get(22).getStartLine());
		assertEquals(1, mdList.get(22).getStartColumn());
		assertEquals(1500, mdList.get(22).getEndLine());
		assertEquals(1, mdList.get(22).getEndColumn());

		assertEquals("heap_tuple_from_minimal_tuple", mdList.get(23).getName());
		assertEquals("heaptuple.c", mdList.get(23).getModifiedName());
		assertEquals("(MinimalTuple mtup)", mdList.get(23).getParams());
		assertEquals("(MinimalTuple)", mdList.get(23).getParamTypeOnly(false));
		assertEquals(1511, mdList.get(23).getStartLine());
		assertEquals(1, mdList.get(23).getStartColumn());
		assertEquals(1525, mdList.get(23).getEndLine());
		assertEquals(1, mdList.get(23).getEndColumn());

		assertEquals("minimal_tuple_from_heap_tuple", mdList.get(24).getName());
		assertEquals("heaptuple.c", mdList.get(24).getModifiedName());
		assertEquals("(HeapTuple htup)", mdList.get(24).getParams());
		assertEquals("(HeapTuple)", mdList.get(24).getParamTypeOnly(false));
		assertEquals(1533, mdList.get(24).getStartLine());
		assertEquals(1, mdList.get(24).getStartColumn());
		assertEquals(1545, mdList.get(24).getEndLine());
		assertEquals(1, mdList.get(24).getEndColumn());
	}
}
