



extern char* vms__StdioReadline(FILE *sys_stdin, FILE *sys_stdout, char *prompt);


PyThreadState* _PyOS_ReadlineTState;

static PyThread_type_lock _PyOS_ReadlineLock = NULL;

int (*PyOS_InputHook)(void) = NULL;

int Py_RISCOSWimpFlag;



static int
my_fgets(char *buf, int len, FILE *fp)
{
	char *p;
	for (;;) {
		if (PyOS_InputHook != NULL)
			(void)(PyOS_InputHook)();
		errno = 0;
		p = fgets(buf, len, fp);
		if (p != NULL)
			return 0; 
		
		if (GetLastError()==ERROR_OPERATION_ABORTED) {
			
			Sleep(1);
			if (PyOS_InterruptOccurred()) {
				return 1; 
			}
			
		}
		if (feof(fp)) {
			return -1; 
		}
		if (errno == EINTR) {
			int s;
			PyEval_RestoreThread(_PyOS_ReadlineTState);
			s = PyErr_CheckSignals();
			PyEval_SaveThread();
			if (s < 0) {
				return 1;
			}
		}
		if (PyOS_InterruptOccurred()) {
			return 1; 
		}
		return -2; 
	}
	
}




char *
PyOS_StdioReadline(FILE *sys_stdin, FILE *sys_stdout, char *prompt)
{
	size_t n;
	char *p;
	n = 100;
	if ((p = (char *)PyMem_MALLOC(n)) == NULL)
		return NULL;
	fflush(sys_stdout);
	if (prompt)
		fprintf(stderr, "%s", prompt);
	if (prompt) {
		if(Py_RISCOSWimpFlag)
			fprintf(stderr, "\x0cr%s\x0c", prompt);
		else
			fprintf(stderr, "%s", prompt);
	}
	fflush(stderr);
	switch (my_fgets(p, (int)n, sys_stdin)) {
	case 0: 
		break;
	case 1: 
		PyMem_FREE(p);
		return NULL;
	case -1: 
	case -2: 
	default: 
		*p = '\0';
		break;
	}
	n = strlen(p);
	while (n > 0 && p[n-1] != '\n') {
		size_t incr = n+2;
		p = (char *)PyMem_REALLOC(p, n + incr);
		if (p == NULL)
			return NULL;
		if (incr > INT_MAX) {
			PyErr_SetString(PyExc_OverflowError, "input line too long");
		}
		if (my_fgets(p+n, (int)incr, sys_stdin) != 0)
			break;
		n += strlen(p+n);
	}
	return (char *)PyMem_REALLOC(p, n+1);
}




char *(*PyOS_ReadlineFunctionPointer)(FILE *, FILE *, char *);




char *
PyOS_Readline(FILE *sys_stdin, FILE *sys_stdout, char *prompt)
{
	char *rv;

	if (_PyOS_ReadlineTState == PyThreadState_GET()) {
		PyErr_SetString(PyExc_RuntimeError,
				"can't re-enter readline");
		return NULL;
	}
	

	if (PyOS_ReadlineFunctionPointer == NULL) {
                PyOS_ReadlineFunctionPointer = vms__StdioReadline;
                PyOS_ReadlineFunctionPointer = PyOS_StdioReadline;
	}
	
	if (_PyOS_ReadlineLock == NULL) {
		_PyOS_ReadlineLock = PyThread_allocate_lock();		
	}

	_PyOS_ReadlineTState = PyThreadState_GET();
	Py_BEGIN_ALLOW_THREADS
	PyThread_acquire_lock(_PyOS_ReadlineLock, 1);

        
        if (!isatty (fileno (sys_stdin)) || !isatty (fileno (sys_stdout)))
                rv = PyOS_StdioReadline (sys_stdin, sys_stdout, prompt);
        else
                rv = (*PyOS_ReadlineFunctionPointer)(sys_stdin, sys_stdout,
                                                     prompt);
	Py_END_ALLOW_THREADS

	PyThread_release_lock(_PyOS_ReadlineLock);

	_PyOS_ReadlineTState = NULL;

	return rv;
}
